from Products.CMFCore.TypesTool import  FactoryTypeInformation
from Products.CMFCore.DirectoryView import addDirectoryViews, registerDirectory, createDirectoryView, manage_listAvailableDirectories
from Products.CMFCore.utils import getToolByName, minimalpath
from Products.CMFTypes.debug import log, log_exc
from Products.CMFTypes.utils import findDict
from Products.CMFTypes import types_globals
from OFS.ObjectManager import BadRequestException
from Globals import package_home
import sys, traceback, os

SKIN_NAME = "cmft"

def install_tools(self, out):
    if not hasattr(self, "generator_tool"):
        addTool = self.manage_addProduct['CMFTypes'].manage_addTool
        addTool('CMFTypes Generator Tool')
    
    if not hasattr(self, "content_tool"):
        addTool = self.manage_addProduct['CMFTypes'].manage_addTool
        addTool('CMFTypes Content Tool')

    #and the tool uses an index
    catalog = getToolByName(self, 'portal_catalog')
    try:
        catalog.addIndex('UID', 'FieldIndex', extra=None)
    except:
        pass

    try:
        if not 'UID' in catalog.schema():
            catalog.addColumn('UID')
    except:
        print >> out, ''.join(traceback.format_exception(*sys.exc_info()))
        print >> out, "Problem updating catalog for UIDs"

    #reindex the objects already in the system
    #this might be an 'update' and not an install
    ct = getToolByName(self, "content_tool")
    ct.index()
    try:
        catalog.manage_reindexIndex(ids=('UID',))
    except:
        pass
    

def install_subskin(self, out, globals=types_globals, product_skins_dir='skins'):
    skinstool=getToolByName(self, 'portal_skins')

    fullProductSkinsPath = os.path.join(package_home(globals), product_skins_dir)
    productSkinsPath = minimalpath(fullProductSkinsPath)
    registered_directories = manage_listAvailableDirectories()
    if productSkinsPath not in registered_directories:
        registerDirectory(product_skins_dir, globals)
    try:
        addDirectoryViews(skinstool, product_skins_dir, globals)
    except BadRequestException, e:
        pass  # directory view has already been added

    files = os.listdir(fullProductSkinsPath)
    for productSkinName in files:
        if os.path.isdir(os.path.join(fullProductSkinsPath, productSkinName)) \
               and productSkinName != 'CVS':
            for skinName in skinstool.getSkinSelections():
                path = skinstool.getSkinPath(skinName)
                path = [i.strip() for i in  path.split(',')]
                try:
                    if productSkinName not in path:
                        path.insert(path.index('custom') +1, productSkinName)
                except ValueError:
                    if productSkinName not in path:
                        path.append(productSkinName)  
                path = ','.join(path)
                skinstool.addSkinSelection(skinName, path)


def install_types(self, out, types, package_name):
    typesTool = getToolByName(self, 'portal_types')
    for type in types:
        try:
            typesTool._delObject(type.__name__)
        except:
            pass
        typesTool.manage_addTypeInformation(FactoryTypeInformation.meta_type,
                                            id=type.__name__,
                                            typeinfo_name="%s: %s" %(package_name, type.__name__))
        # set the human readable title explicitly
        t = getattr(typesTool, type.__name__, None)
        if t:
            t.title = type.portal_type

def install_validation(self, out, types):
    form_tool = getToolByName(self, 'portal_form')
    type_tool = getToolByName(self, 'portal_types')

    # No validation on references
    form_tool.setValidators('reference_edit', [])
    
    for type in types:
        if 'validation' in type.installMode:
            ti = getattr(type_tool, type.__name__)
            actions = ti._actions
            action = findDict(actions, 'id', 'metadata')
            if action:
                form_tool.setValidators(action['action'], [])
    
            #For now we have to hardcode this till we record more on the GeneratorTool
            key = type.__name__.lower()
            form_tool.setValidators("%s_edit_form" % key, ['validate_id', 'validate_%s_edit' % key])
        


def install_navigation(self, out, types):
    nav_tool = getToolByName(self, 'portal_navigation')
    type_tool = getToolByName(self, 'portal_types')
    
    #Generic Edit
    script = "content_edit" 
    nav_tool.addTransitionFor('default', script , 'failure', 'action:edit')
    nav_tool.addTransitionFor('default', script, 'success', 'action:view')

    #And References
    nav_tool.addTransitionFor('default', 'reference_edit', 'success', 'pasteReference')
    nav_tool.addTransitionFor('default', 'reference_edit', 'failure', 'url:reference_edit')

    #Type Specific
    for type in types:
        if 'navigation' in type.installMode:
            ti = getattr(type_tool, type.__name__)
            actions = ti._actions
    
            action = findDict(actions, 'id', 'metadata')
            if action:
                action = action['action'][action['action'].find('/')+1:]
                nav_tool.addTransitionFor('default', action, 'failure', action)
                nav_tool.addTransitionFor('default', action,
                                          'success', 'script:%s' % script)
    
            action = findDict(actions, 'id', 'edit')
            if action:
                # we know that edit might be hacked, refactoring gentool
                # will fix this problem
                action = action['action'][action['action'].find('/')+1:]
                nav_tool.addTransitionFor('default', action, 'success',
                                          'script:%s' % script) 
                nav_tool.addTransitionFor('default', action, 'failure', action)





def install_actions(self, out, types):
    typesTool = getToolByName(self, 'portal_types')
    for type in types:
        if 'actions' in type.installMode:
            typeInfo = getattr(typesTool, type.__name__)
            if hasattr(type,'actions'):
                #Look for each action we define in type.actions
                #in typeInfo.action replacing it if its there and
                #just adding it if not
                new = list(typeInfo._actions)
                for action in type.actions:
                    hit = findDict(typeInfo._actions, 'id', action['id'])
                    if hit:
                        hit.update(action)
                    else:
                        new.append(action)
                typeInfo._actions = tuple(new)
                typeInfo._p_changed = 1
    
            if hasattr(type,'factory_type_information'):
                typeInfo.__dict__.update(type.factory_type_information)
                typeInfo._p_changed = 1


def isPloneSite(self):
    # we should just define a single attr for this
    if self.__class__.__name__ == "PloneSite":
        return 1
    return 0

def installTypes(self,
                 out,
                 types,
                 package_name, 
                 globals=types_globals,
                 product_skins_dir='skins'):
    """Use this for your site with your types"""
    install_tools(self, out)
    install_subskin(self, out, globals, product_skins_dir)
    install_types(self, out, types, package_name)

    install_actions(self, out, types)
    
    if isPloneSite(self):
        install_validation(self, out, types)
        install_navigation(self, out, types)
        

